<?php

namespace Seb\Baltic\Controller\Token;

class Delete extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var \Seb\Baltic\Api\Data\TokenRepositoryInterface
     */
    protected $tokenRepository;

    /**
     * @var \Seb\Baltic\Model\Token\TokenManagement
     */
    protected $tokenManager;

    /**
     * @var \Seb\Baltic\Logger\Logger
     */
    protected $logger;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \Seb\Baltic\Api\Data\TokenRepositoryInterface $tokenRepository
     * @param \Seb\Baltic\Model\Token\TokenManagement $tokenManager
     * @param \Seb\Baltic\Logger\Logger $logger
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Seb\Baltic\Api\Data\TokenRepositoryInterface $tokenRepository,
        \Seb\Baltic\Model\Token\TokenManagement $tokenManager,
        \Seb\Baltic\Logger\Logger $logger
    ) {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->tokenRepository = $tokenRepository;
        $this->tokenManager = $tokenManager;
        $this->logger = $logger;

        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        $id = $this->_request->getParam('id');
        $result = $this->resultJsonFactory->create();
        $token  = null;

        try {
            $token = $this->tokenRepository->getById($id);
        } catch (\Exception $e) {
            $this->logger->error('Exception while retrieving token', [$e->getMessage(), $e->getTrace()]);
            return $result->setData(['success' => false]);
        }

        if ($this->tokenManager->isCustomersToken($token)) {
            try {
                $this->tokenRepository->delete($token);
                $result->setData(['success' => true]);
            } catch (\Exception $e) {
                $this->logger->error('Exception while deleting token', [$e->getMessage(), $e->getTrace()]);
                $result->setData(['success' => false]);
            }
        } else {
            $result->setData(['success' => false]);
        }

        return $result;
    }
}