<?php

namespace Seb\Baltic\Helper;

use Magento\Framework\Locale\Resolver as LocaleResolver;

class Locale
{
    /**
     * @var string
     */
    private $defaultLocale = 'en';

    /**
     * @var LocaleResolver
     */
    private $localeResolver;

    /**
     * Locale constructor.
     *
     * @param LocaleResolver $localeResolver
     */
    public function __construct(
        LocaleResolver $localeResolver,
    ) {
        $this->localeResolver = $localeResolver;
    }

    /**
     * Get the locale.
     *
     * @return string
     */
    public function getLocale()
    {
        $code = strtolower($this->localeResolver->getLocale());

        foreach ($this->getSupportedLocales() as $locale => $codes) {
            if (in_array($code, $codes)) {
                return $locale;
            }
        }

        return $this->defaultLocale;
    }

    /**
     * Get the supported locales by the SDK.
     *
     * @return array[]
     */
    public function getSupportedLocales()
    {
        return [
            'cz' => ['cz', 'cs', 'cs_cz'],
            'da' => ['da', 'da_dk'],
            'de' => ['de', 'de_de', 'de_at', 'de_ch'],
            'en' => ['en', 'en_us', 'en_au', 'en_ca', 'en_nz', 'en_gb'],
            'es' => ['es', 'es_es', 'es_ar', 'es_mx'],
            'et' => ['et', 'et_ee'],
            'fi' => ['fi', 'fi_fi'],
            'fr' => ['fr', 'fr_fr', 'fr_ca'],
            'hu' => ['hu', 'hu_hu'],
            'it' => ['it', 'it_it'],
            'lt' => ['lt', 'lt_lt'],
            'lv' => ['lv', 'lv_lv'],
            'nl' => ['nl', 'nl_nl', 'nl_be'],
            'no' => ['no', 'nb_no', 'nn_no'],
            'pl' => ['pl', 'pl_pl'],
            'pt' => ['pt', 'pt-br', 'pt-pt', 'pt_br', 'pt_pt'],
            'ru' => ['ru', 'ru_ru'],
            'sk' => ['sk', 'sk_sk'],
            'sv' => ['sv', 'sv_se'],
            'uk' => ['uk', 'uk_ua'],
        ];
    }
}
