<?php

namespace Seb\Baltic\Model;

class CheckoutData
{
    /**
     * @var \Seb\Baltic\Model\Gateway
     */
    protected $api;

    /**
     * @var \Magento\Checkout\Model\Cart
     */
    protected $cart;

    /**
     * @var \Magento\Directory\Model\Config\Source\Country
     */
    protected $countryList;

    /**
     * @var \Seb\Baltic\Model\Config
     */
    protected $config;

    /**
     * @param \Seb\Baltic\Model\Gateway $api
     * @param \Magento\Checkout\Model\Cart $cart
     * @param \Magento\Directory\Model\Config\Source\Country $countryList
     * @param \Seb\Baltic\Model\Config $config
     */
    public function __construct(
        \Seb\Baltic\Model\Gateway $api,
        \Magento\Checkout\Model\Cart $cart,
        \Magento\Directory\Model\Config\Source\Country $countryList,
        \Seb\Baltic\Model\Config $config
    ) {
        $this->api = $api;
        $this->cart = $cart;
        $this->countryList = $countryList;
        $this->config = $config;
    }

    /**
     * @return array
     */
    public function getCheckoutData()
    {
        $amount = $this->getAmount();
        $paymentMethodsData = $this->api->getPaymentMethodsData($amount);
        return [
            'methods' => $paymentMethodsData['payment_methods'],
            'countries' => $this->getCountries($paymentMethodsData['supported_countries']),
            'amount' => $amount
        ];
    }

    /**
     * @return float
     */
    public function getAmount()
    {
        return $this->cart->getQuote() ? round($this->cart->getQuote()->getBaseGrandTotal(), 2) : 0;
    }

    /**
     * @return array
     */
    public function getCountries($supportedCountries = null)
    {
        $data = [];
        $codes = $supportedCountries ?? $this->config->getSupportedCountries();
        foreach ($this->countryList->toOptionArray() as $countryData) {
            if (isset($countryData['value']) && in_array($countryData['value'], $codes)) {
                $data[] = $countryData;
            }
        }
        return $data;
    }
}
