<?php

namespace Seb\Baltic\Model;

use Magento\Framework\Exception\LocalizedException;

class GuestPaymentInformationAdapter implements \Seb\Baltic\Api\GuestPaymentInformationAdapterInterface
{
    /**
     * @var \Magento\Checkout\Api\GuestPaymentInformationManagementInterface
     */
    protected $guestPaymentInformationManagement;

    /**
     * @var \Seb\Baltic\Model\Gateway
     */
    protected $api;

    /**
     * @var \Seb\Baltic\Logger\Logger
     */
    protected $logger;

    /**
     * @param \Magento\Checkout\Api\GuestPaymentInformationManagementInterface $guestPaymentInformationManagement
     * @param \Seb\Baltic\Model\Gateway $api
     * @param \Seb\Baltic\Logger\Logger $logger
     */
    public function __construct(
        \Magento\Checkout\Api\GuestPaymentInformationManagementInterface $guestPaymentInformationManagement,
        \Seb\Baltic\Model\Gateway $api,
        \Seb\Baltic\Logger\Logger $logger
    ) {
        $this->guestPaymentInformationManagement = $guestPaymentInformationManagement;
        $this->api = $api;
        $this->logger = $logger;
    }

    /**
     * {@inheritDoc}
     */
    public function adaptPlaceOrder(
        $cartId,
        $email,
        $methodId,
        \Magento\Quote\Api\Data\PaymentInterface $paymentMethod,
        \Magento\Quote\Api\Data\AddressInterface $billingAddress = null
    ) {
        try {
            $orderId = $this->guestPaymentInformationManagement->savePaymentInformationAndPlaceOrder(
                $cartId, $email, $paymentMethod, $billingAddress
            );

            return json_encode($this->api->initPayment($orderId, $methodId));
        } catch (\Exception $e) {
            $this->logger->error('An error occured while placing the order', [$e->getMessage(), $e->getTrace()]);
            throw new LocalizedException(__($e->getMessage()));
        }
    }
}
