<?php

namespace Seb\Baltic\Model;

use Seb\Baltic\Helper\Locale;
use Seb\Baltic\Model\Gateway;
use Seb\Baltic\Model\Method\AlternativeMethod;
use Seb\Baltic\Model\Method\AppleMethod;
use Seb\Baltic\Model\Method\BankMethod;
use Seb\Baltic\Model\Method\CardMethod;
use Seb\Baltic\Model\Method\GoogleMethod;
use Seb\Baltic\Model\Token\TokenManagement;
use Magento\Customer\Model\Url;

class PaymentConfigProvider implements \Magento\Checkout\Model\ConfigProviderInterface
{
    /**
     * @var Url
     */
    protected $customerUrl;

    /**
     * @var TokenManagement
     */
    protected $tokenManager;

    /**
     * @var CheckoutData
     */
    protected $checkoutData;

    /**
     * @var Config
     */
    protected $config;

    /**
     * @var Locale
     */
    protected $locale;

    /**
     * BalticConfigProvider constructor.
     *
     * @param CheckoutData $checkoutData
     * @param Config $config
     * @param Locale $locale
     * @param TokenManagement $tokenManager
     * @param Url $customerUrl
     */
    public function __construct(
        CheckoutData $checkoutData,
        Config $config,
        Locale $locale,
        TokenManagement $tokenManager,
        Url $customerUrl
    ) {
        $this->checkoutData = $checkoutData;
        $this->config = $config;
        $this->customerUrl = $customerUrl;
        $this->locale = $locale;
        $this->tokenManager = $tokenManager;
    }

    /**
     * @return array
     */
    public function getConfig()
    {
        $checkoutData = $this->checkoutData->getCheckoutData();
        return [
            'payment' => [
                'baltic' => [
                    'account' => $this->config->getProcessingAccount(),
                    'amount' => $checkoutData['amount'],
                    'customerUrl' => $this->customerUrl->getLoginUrl(),
                    'defaultCountry' => $this->config->getDefaultCountry(),
                    'environment' => $this->config->getSdkEnvironment(),
                    'layout' => $this->config->getLayout(),
                    'locale' => $this->locale->getLocale(),
                    'theme' => $this->config->getTheme(),
                    'username' => $this->config->getApiUser(),
                ]
            ]
        ];
    }

    /**
     * @return array
     */
    protected function getCards()
    {
        $cards = [];
        $logos = $this->getLogos();
        foreach ($this->tokenManager->getListByCustomerSession() as $token) {
            $cards[] = [
                'id' => $token['id'],
                'label' =>
                    sprintf("**** **** **** %s (%s %s/%s)",
                        $token['cc_last_four_digits'],
                        __('expires'),
                        $token['cc_month'],
                        $token['cc_year']
                    ),
                'image' => $logos[$token['cc_type']]['image'],
                'type' => str_replace('_', '-', $token['cc_type']),
            ];
        }

        if (!empty($cards)) {
            $newCard = [
                'id' => Config::NEW_CARD_ID,
                'label' => __('New card'),
                'image' => '',
                'type' => 'new-card'
            ];

            $cards[] = $newCard;
        }

        return $cards;
    }
}
