<?php

namespace Seb\Baltic\Model;

use Magento\Framework\Exception\LocalizedException;

class PaymentInformationAdapter implements \Seb\Baltic\Api\PaymentInformationAdapterInterface
{
    /**
     * @var \Magento\Checkout\Api\PaymentInformationManagementInterface
     */
    protected $paymentInformationManagement;

    /**
     * @var \Seb\Baltic\Model\Gateway
     */
    protected $api;

    /**
     * @var \Seb\Baltic\Logger\Logger
     */
    protected $logger;

    /**
     * @param \Magento\Checkout\Api\PaymentInformationManagementInterface $paymentInformationManagement
     * @param \Seb\Baltic\Model\Gateway $api
     * @param \Seb\Baltic\Logger\Logger $logger
     */
    public function __construct(
        \Magento\Checkout\Api\PaymentInformationManagementInterface $paymentInformationManagement,
        \Seb\Baltic\Model\Gateway $api,
        \Seb\Baltic\Logger\Logger $logger
    ) {
        $this->paymentInformationManagement = $paymentInformationManagement;
        $this->api = $api;
        $this->logger = $logger;
    }

    /**
     * {@inheritDoc}
     */
    public function adaptPlaceOrder(
        $cartId,
        $tokenId,
        $methodId,
        \Magento\Quote\Api\Data\PaymentInterface $paymentMethod,
        \Magento\Quote\Api\Data\AddressInterface $billingAddress = null
    ) {
        try {
            $orderId = $this->paymentInformationManagement->savePaymentInformationAndPlaceOrder(
                $cartId, $paymentMethod, $billingAddress
            );

            return json_encode($this->api->initPayment($orderId, $methodId, $tokenId));
        } catch (\Exception $e) {
            $this->logger->error('An error occured while placing the order', [$e->getMessage(), $e->getTrace()]);
            throw new LocalizedException(__($e->getMessage()));
        }
    }
}
