define(
    [
        'ko',
        'jquery',
        'mage/translate',
        'Magento_Ui/js/modal/alert',
        'Magento_Checkout/js/view/payment/default',
        'Magento_Checkout/js/model/payment/additional-validators',
        'Magento_Checkout/js/model/quote',
        'Magento_Checkout/js/model/full-screen-loader',
        'Seb_Baltic/js/action/place-order',
        'Seb_Baltic/js/action/place-order-apple',
        'Seb_Baltic/js/model/token-manager',
        'Seb_Baltic/js/model/method-manager',
        'Seb_Baltic/js/model/ui-manager',
        'Seb_Baltic/js/action/handle-redirect',
        'mage/mage'
    ],
    function (
        ko,
        $,
        $t,
        magentoAlert,
        Component,
        additionalValidators,
        quote,
        fullScreenLoader,
        placeOrderAction,
        placeOrderAppleAction,
        tokenManager,
        methodManager,
        uiManager,
        handleRedirect
    ) {
        'use strict';
        var self = this;

        return Component.extend({
            redirectAfterPlaceOrder: false,
            tokenManager: tokenManager(),
            methodManager: methodManager(),
            uiManager: uiManager(),

            selectedCard: ko.observable(null),
            isTokenEnabled: window.checkoutConfig.payment.baltic.isTokenEnabled,
            isLoggedIn: window.checkoutConfig.payment.baltic.customer.isLoggedIn,
            hasTokens: window.checkoutConfig.payment.baltic.customer.cards.length > 0,
            loginUrl: window.checkoutConfig.payment.baltic.customerUrl,
            countryOptions: ko.observableArray(window.checkoutConfig.payment.baltic.countries),
            methodsOrder: ko.observableArray(window.checkoutConfig.payment.baltic.methodsOrder),
            methodTemplates: {
                'card': 'Seb_Baltic/payment/method/card',
                'banks': 'Seb_Baltic/payment/method/bank',
                'alt': 'Seb_Baltic/payment/method/alt',
                'apple': 'Seb_Baltic/payment/method/apple',
                'google': 'Seb_Baltic/payment/method/google'
            },
            defaults: {
                template: 'Seb_Baltic/payment/method-template'
            },
            initialize: function() {
                this._super();
                window.addEventListener('balticCountries', () => {
                    this.updateCountries()
                });
            },
            updateCountries: function () {
                this.countryOptions(window.checkoutConfig.payment.baltic.countries);
            },
            googleButtonInitialized: false,
            validate: function () {
                var billingAddress = quote.billingAddress();

                if (!billingAddress.city || !billingAddress.countryId || !billingAddress.postcode) {
                    magentoAlert({
                        title: $t('Error'),
                        content: $t('Your session has expired'),
                        actions: {
                            always: function () {
                                var url = window.checkoutConfig.checkoutUrl;
                                window.location.replace(url);
                            }
                        }
                    });
                    return false;
                }
                return true;
            },
            placeOrder: function (data, event) {
                var self = this;
                var placeOrder;

                if (event) {
                    event.preventDefault();
                }

                fullScreenLoader.startLoader();
                $('body').trigger('processStart');

                if (self.validate() && additionalValidators.validate()) {
                    var paymentData = self.getData();
                    paymentData.method = self.getMethodCode();

                    placeOrder = placeOrderAction(paymentData, self.messageContainer, self.getTokenData(), self.getMethodId());
                    placeOrder.fail(function () {
                        fullScreenLoader.stopLoader();
                        $('body').trigger('processStop');
                    }).done(self.afterPlaceOrder.bind(self));
                } else {
                    $('body').trigger('processStop');
                }

                fullScreenLoader.stopLoader();
                return true;
            },
            placeOrderApple: function (data, event) {
                var self = this;
                var placeOrderApple;

                if (event) {
                    event.preventDefault();
                }

                fullScreenLoader.startLoader();
                $('body').trigger('processStart');

                if (self.validate() && additionalValidators.validate()) {
                    setTimeout(function() {
                        var paymentData = self.getData();
                        paymentData.method = self.getMethodCode();

                        var response = placeOrderAppleAction(paymentData, self.messageContainer);
                        if(response) {
                            self.afterPlaceOrderApple(response);
                        }
                        $('body').trigger('processStop');
                        fullScreenLoader.stopLoader();
                    }, 500);
                } else {
                    $('body').trigger('processStop');
                    fullScreenLoader.stopLoader();
                }
                return true;
            },
            afterPlaceOrder: function (data) {
                handleRedirect(data, this.uiManager.isCardsSelected()());
            },
            afterPlaceOrderApple: function (response) {
                var data = response[0];
                if (typeof data !== "undefined") {
                    var applePayClient = window.ApplePayClient.initialize({
                        access_token: data.access_token,
                        api_username: data.api_username,
                        merchant_id: data.merchant_id,
                        account_name: data.account_name,
                        env: data.payment_link,
                    });

                    applePayClient.createApplePayPayment({
                        payment_reference: data.payment_reference,
                        country_code: data.country_code,
                        currency_code: data.currency_code,
                        payment_link: data.payment_link,
                        total: {
                            label: data.total_label,
                            amount: data.total_amount
                        }
                    });
                }
            },
            getTokenData: function() {
                var tokenId = null;

                if (this.methodManager.isCardSelected() && this.selectedCard() != null) {
                    tokenId = this.selectedCard();
                }

                return {
                    tokenId: tokenId
                };
            },
            getMethodId: function() {
                return this.methodManager.getSelectedMethod() ? this.methodManager.getSelectedMethod().getId() : '';
            },
            getMethodCode: function() {
                return this.methodManager.getSelectedMethod() ?
                    this.getCode() + '_' + this.methodManager.getSelectedMethod().getType() : this.getCode();
            },
            getCode: function() {
                return 'baltic';
            },
            getLogos: function() {
                return window.checkoutConfig.payment.baltic.logos;
            },
            deleteToken: function(id) {
                fullScreenLoader.startLoader();
                $.ajax({
                    url: window.checkoutConfig.payment.baltic.deleteUrl + 'id/' + id.id,
                    type: 'get',
                    success: function(data) {
                        if (data.success) {
                            $('.card-token-' + id.id).remove();
                        }
                    }
                });
                fullScreenLoader.stopLoader();
            },
            isPlaceOrderAllowed: function() {
                var selectedMethod = this.methodManager.getSelectedMethod();
                var activeType = this.uiManager.getActiveType();
                if (!selectedMethod ||
                    selectedMethod.getType() !== activeType ||
                    !selectedMethod.isVisible()
                ) {
                    return false;
                }
                if (quote.billingAddress() == null) {
                    return false;
                }
                if (
                    this.isLoggedIn && this.isTokenEnabled && this.hasTokens &&
                    this.methodManager.isCardSelected() && !this.selectedCard()
                ) {
                    return false;
                }
                return true;
            },
            isApplePlaceOrderAllowed: function() {
                var activeType = this.uiManager.getActiveType();
                if (activeType !== this.methodManager.getAppleCode()) {
                    return false;
                }
                if (quote.billingAddress() == null) {
                    return false;
                }
                return true;
            },
            isGooglePlaceOrderAllowed: function() {
                var activeType = this.uiManager.getActiveType();
                if (activeType !== this.methodManager.getGoogleCode()) {
                    return false;
                }
                if (quote.billingAddress() == null) {
                    return false;
                }
                return true;
            },
            checkGooglePlaceOrderAllowed: function() {
                var isAllowed = this.isGooglePlaceOrderAllowed();

                if(isAllowed) {
                    if(!this.googleButtonInitialized) {
                        if(typeof GooglePayClient !== 'undefined') {
                            const googleInitialize = new CustomEvent("baltic.google-initialize", {
                                detail: {
                                    method: this,
                                    methodManager: this.methodManager
                                }
                            });
                            window.dispatchEvent(googleInitialize);
                        }
                        this.googleButtonInitialized = true;
                    }
                    $('#google-pay-container button').removeAttr('disabled');
                } else {
                    $('#google-pay-container button').attr('disabled','disabled');
                }

                return isAllowed;
            },
            onCountryChange: function() {
                this.methodManager.selectCountry(this.methodManager.selectedCountry());
            },
            selectPaymentMethod: function (type) {
                this.uiManager.setActiveType(type);
                return this._super();
            },
            getMethodTemplate: function(code) {
                return this.methodTemplates[code];
            },
            /**
             * States wheter the radio button should be displayed.
             *
             * @returns {boolean}
             */
            shouldDisplayRadioButton() {
                return this.methodManager.getVisibleMethodsCount() !== 1 ||
                    this.methodManager.getVisibleMethodsCount();
            }
        });
    }
);
