define(
    [
        'jquery',
        'mage/translate',
        'Magento_Ui/js/modal/alert',
        'Magento_Checkout/js/view/payment/default',
        'Magento_Checkout/js/model/quote',
        'Magento_Customer/js/model/customer',
        'Magento_Checkout/js/model/full-screen-loader',
        'Magento_Checkout/js/model/error-processor',
        'Seb_Baltic/js/action/place-order-sdk',
        'Seb_Baltic/js/helper/email',
        'Seb_Baltic/js/helper/validate',
        'Seb_Baltic/js/model/cart-data',
    ],
    function (
        $,
        $t,
        magentoAlert,
        Component,
        quote,
        customer,
        fullScreenLoader,
        errorProcessor,
        placeOrderAction,
        emailHelper,
        validateHelper,
        cartData
    ) {
        'use strict';

        return Component.extend({
            defaults: {
                template: 'Seb_Baltic/payment/sdk',
                billingData: {
                    amount: null,
                    countryId: null,
                    email: null,
                },
                config: null,
                paymentElement: null,
            },
            redirectAfterPlaceOrder: false,

            initialize: function() {
                this._super();

                this.config = window.checkoutConfig.payment[this.getCode()];
                this.billingData.email = cartData.getEmailAddress();
                this.billingData.countryId = cartData.getBillingCountry(this.config.defaultCountry);
                this.billingData.amount = parseFloat(this.config.amount);

                this.initSubscribers();

                return this;
            },

            placeOrder: async function(data, event) {
                if (!this.isWallet()) {
                    fullScreenLoader.startLoader();
                }

                if (event) {
                    event.preventDefault();
                }

                if (validateHelper.validate()) {
                    fullScreenLoader.stopLoader();

                    const submitResponse = await this.paymentElement.submit();
                    if (!submitResponse.error) {
                        const paymentData = this.getData();
                        if (!paymentData.additional_data) {
                            paymentData.additional_data = {};
                        }
                        paymentData.additional_data.selectedMethod = this.paymentElement.selectedPaymentMethod();

                        try {
                            const placeOrderResponse = placeOrderAction(paymentData, this.messageContainer);

                            await this.paymentElement.confirm(JSON.parse(placeOrderResponse));
                        } catch (e) {}
                    }
                }

                fullScreenLoader.stopLoader();

                return true;
            },

            isWallet: function() {
                const paymentMethod = this.paymentElement.selectedPaymentMethod();

                return paymentMethod === 'google_pay' || paymentMethod === 'apple_pay';
            },

            initSdk: function() {
                if (!window.EveryPay) {
                    return;
                }

                const config = this.config;

                const everyPay = new window.EveryPay({
                    account: config.account,
                    username: config.username,
                });

                const stylingOptions = Object.freeze({
                    theme: config.theme,
                    layout: config.layout,
                });

                const secureElementOptions = Object.freeze({
                    stylingOptions: stylingOptions,
                    amount: this.billingData.amount,
                    email: this.billingData.email,
                    environment: config.environment,
                    locale: config.locale,
                    preferredCountry: this.billingData.countryId,
                    username: config.username,
                });

                const paymentElementOptions = Object.freeze({
                    element: 'payment',
                });

                this.paymentElement = everyPay
                    .secureElements(secureElementOptions)
                    .build(paymentElementOptions);

                this.paymentElement
                    .mount('#balticPaymentElement')
                    .catch(function onPaymentElementMountError(err) {
                        console.error(err);
                    });
            },

            removeSdk: function() {
                document.querySelector('#balticPaymentElement').innerHTML = '';
            },

            removeAndInitSdk: function() {
                this.removeSdk();
                this.initSdk();
            },

            initSubscribers: function() {
                const self = this;

                quote.totals.subscribe(function(totals) {
                    if (totals && totals.base_grand_total && totals.base_grand_total !== self.billingData.amount) {
                        self.billingData.amount = totals.base_grand_total;

                        self.removeAndInitSdk();
                    }
                });

                quote.billingAddress.subscribe(function(address) {
                    if (address && address.countryId && address.countryId !== self.billingData.countryId) {
                        self.billingData.countryId = address.countryId;

                        self.removeAndInitSdk();
                    }

                    const newEmail = cartData.getEmailAddress();
                    if (newEmail && emailHelper.validateEmail(newEmail) && newEmail !== self.billingData.email) {
                        self.billingData.email = newEmail;

                        self.removeAndInitSdk();
                    }
                });

                const emailObservable = cartData.getObservableEmailAddress();
                emailObservable.subscribe(function(email) {
                    if (email && emailHelper.validateEmail(email) && email !== self.billingData.email) {
                        self.billingData.email = email;

                        self.removeAndInitSdk();
                    }
                })
            }
        });
    }
);
